@@ -1,5 +1,5 @@
 /* DropTargetContext.java --
-   Copyright (C) 2002, 2003, 2004  Free Software Foundation
+   Copyright (C) 2002, 2003, 2004, 2006,  Free Software Foundation
 
 This file is part of GNU Classpath.
 
@@ -41,6 +41,7 @@ import java.awt.Component;
 import java.awt.datatransfer.DataFlavor;
 import java.awt.datatransfer.Transferable;
 import java.awt.datatransfer.UnsupportedFlavorException;
+import java.awt.dnd.peer.DropTargetContextPeer;
 import java.io.IOException;
 import java.io.Serializable;
 import java.util.Arrays;
@@ -59,23 +60,23 @@ public class DropTargetContext implements Serializable
     protected boolean isLocal;
     protected Transferable transferable;
 
-    TransferableProxy (Transferable t, boolean local)
+    TransferableProxy(Transferable t, boolean local)
     {
       this.transferable = t;
       this.isLocal = local;
     }
     
-    public DataFlavor[] getTransferDataFlavors ()
+    public DataFlavor[] getTransferDataFlavors()
     {
-      return transferable.getTransferDataFlavors ();
+      return transferable.getTransferDataFlavors();
     }
 
-    public boolean isDataFlavorSupported (DataFlavor flavor)
+    public boolean isDataFlavorSupported(DataFlavor flavor)
     {
-      return transferable.isDataFlavorSupported (flavor);
+      return transferable.isDataFlavorSupported(flavor);
     }
 
-    public Object getTransferData (DataFlavor flavor)
+    public Object getTransferData(DataFlavor flavor)
       throws UnsupportedFlavorException, IOException
     {
       return transferable.getTransferData (flavor);
@@ -84,35 +85,35 @@ public class DropTargetContext implements Serializable
 
   private DropTarget dropTarget;
   private int targetActions;
-  private java.awt.dnd.peer.DropTargetContextPeer dtcp;
+  private DropTargetContextPeer dtcp;
 
   // package private
-  DropTargetContext (DropTarget dropTarget)
+  DropTargetContext(DropTarget dropTarget)
   {
     this.dropTarget = dropTarget;
   }
 
-  public DropTarget getDropTarget ()
+  public DropTarget getDropTarget()
   {
     return dropTarget;
   }
 
-  public Component getComponent ()
+  public Component getComponent()
   {
-    return dropTarget.getComponent ();
+    return dropTarget.getComponent();
   }
 
-  public void addNotify (java.awt.dnd.peer.DropTargetContextPeer dtcp)
+  public void addNotify(DropTargetContextPeer dtcp)
   {
     this.dtcp = dtcp;
   }
 
-  public void removeNotify ()
+  public void removeNotify()
   {
     this.dtcp = null;
   }
 
-  protected void setTargetActions (int actions)
+  protected void setTargetActions(int actions)
   {
     targetActions = actions;
   }
@@ -127,45 +128,51 @@ public class DropTargetContext implements Serializable
    *
    * @exception InvalidDnDOperationException If a drop is not outstanding.
    */
-  public void dropComplete (boolean success)
+  public void dropComplete(boolean success)
   {
-    // FIXME: implement this
+    if (dtcp != null)
+      dtcp.dropComplete(success);
   }
 
-  protected void acceptDrag (int dragOperation)
+  protected void acceptDrag(int dragOperation)
   {
-    // FIXME: implement this
+    if (dtcp != null)
+      dtcp.acceptDrag(dragOperation);
   }
 
-  protected void rejectDrag ()
+  protected void rejectDrag()
   {
-    // FIXME: implement this
+    if (dtcp != null)
+      dtcp.rejectDrag();
   }
 
-  protected void acceptDrop (int dropOperation)
+  protected void acceptDrop(int dropOperation)
   {
-    // FIXME: implement this
+    if (dtcp != null)
+      dtcp.acceptDrop(dropOperation);
   }
 
-  protected void rejectDrop ()
+  protected void rejectDrop()
   {
-    // FIXME: implement this
+    if (dtcp != null)
+      dtcp.rejectDrop();    
   }
 
-  protected DataFlavor[] getCurrentDataFlavors ()
+  protected DataFlavor[] getCurrentDataFlavors()
   {
-    // FIXME: implement this
+    if (dtcp != null)
+      dtcp.getTransferDataFlavors();
     return null;
   }
 
-  protected List getCurrentDataFlavorsAsList ()
+  protected List getCurrentDataFlavorsAsList()
   {
-    return Arrays.asList (getCurrentDataFlavors ());
+    return Arrays.asList(getCurrentDataFlavors());
   }
 
-  protected boolean isDataFlavorSupported (DataFlavor flavor)
+  protected boolean isDataFlavorSupported(DataFlavor flavor)
   {
-    return getCurrentDataFlavorsAsList ().contains (flavor);
+    return getCurrentDataFlavorsAsList().contains(flavor);
   }
 
   /**
@@ -173,14 +180,17 @@ public class DropTargetContext implements Serializable
    *
    * @exception InvalidDnDOperationException If a drag is not outstanding.
    */
-  protected Transferable getTransferable() throws InvalidDnDOperationException
+  protected Transferable getTransferable() 
+    throws InvalidDnDOperationException
   {
-    // FIXME: implement this
+    // FIXME: Implement this
+    if (dtcp != null)
+      return dtcp.getTransferable();
     return null;
   }
 
   protected Transferable createTransferableProxy(Transferable t, boolean local)
   {
-    return new TransferableProxy (t, local);
+    return new TransferableProxy(t, local);
   }
 } // class DropTargetContext
