@@ -1,5 +1,5 @@
 /* GeneralPath.java -- represents a shape built from subpaths
-   Copyright (C) 2002, 2003, 2004 Free Software Foundation
+   Copyright (C) 2002, 2003, 2004, 2006 Free Software Foundation
 
 This file is part of GNU Classpath.
 
@@ -65,8 +65,8 @@ import java.awt.Shape;
  * &#x2019;up&#x2019;
  * direction, one in the &#x2019;down&#x2019; direction) Point <b>B</b> in 
  * the image is inside (one intersection &#x2019;down&#x2019;)
- * Point <b>C</b> in the image is outside (two intersections 
- * &#x2019;down&#x2019;)
+ * Point <b>C</b> in the image is inside (two intersections in the 
+ * &#x2019;down&#x2019; direction)
  *
  * @see Line2D
  * @see CubicCurve2D
@@ -79,8 +79,16 @@ import java.awt.Shape;
  */
 public final class GeneralPath implements Shape, Cloneable
 {
-  public static final int WIND_EVEN_ODD = PathIterator.WIND_EVEN_ODD;
-  public static final int WIND_NON_ZERO = PathIterator.WIND_NON_ZERO;
+  // WORKAROUND for gcj 4.0.x (x < 3)
+  // fully qualify PathIterator constants.
+
+  /** Same constant as {@link PathIterator#WIND_EVEN_ODD}. */
+  public static final int WIND_EVEN_ODD
+    = java.awt.geom.PathIterator.WIND_EVEN_ODD;
+
+  /** Same constant as {@link PathIterator.WIND_NON_ZERO}. */
+  public static final int WIND_NON_ZERO
+    = java.awt.geom.PathIterator.WIND_NON_ZERO;
 
   /** Initial size if not specified. */
   private static final int INIT_SIZE = 10;
@@ -239,10 +247,12 @@ public final class GeneralPath implements Shape, Cloneable
 
   /**
    * Closes the current subpath by drawing a line
-   * back to the point of the last moveTo.
+   * back to the point of the last moveTo, unless the path is already closed.
    */
   public void closePath()
   {
+    if (index >= 1 && types[index - 1] == PathIterator.SEG_CLOSE)
+      return;
     ensureSize(index + 1);
     types[index] = PathIterator.SEG_CLOSE;
     xpoints[index] = xpoints[subpath];
@@ -558,7 +568,8 @@ public final class GeneralPath implements Shape, Cloneable
      * Constructs a new iterator for enumerating the segments of a
      * GeneralPath.
      *
-     * @param at an affine transformation for projecting the returned
+     * @param path the path to enumerate
+     * @param transform an affine transformation for projecting the returned
      * points, or <code>null</code> to return the original points
      * without any mapping.
      */
