@@ -135,6 +135,8 @@ Java_gnu_java_awt_peer_gtk_GtkToolkit_gtkInit (JNIEnv *env,
 
   gtkgenericpeer = (*env)->FindClass(env, "gnu/java/awt/peer/gtk/GtkGenericPeer");
 
+  gtkgenericpeer = (*env)->NewGlobalRef(env, gtkgenericpeer);
+
   printCurrentThreadID = (*env)->GetStaticMethodID (env, gtkgenericpeer,
                                                     "printCurrentThread", "()V");
  
@@ -160,9 +162,7 @@ Java_gnu_java_awt_peer_gtk_GtkToolkit_gtkInit (JNIEnv *env,
   XSynchronize (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), True);
 #endif
 
-  gdk_rgb_init ();
-  gtk_widget_set_default_colormap (gdk_rgb_get_cmap ());
-  gtk_widget_set_default_visual (gdk_rgb_get_visual ());
+  gtk_widget_set_default_colormap (gdk_rgb_get_colormap ());
 
   /* Make sure queued calls don't get sent to GTK/GDK while 
      we're shutting down. */
@@ -185,10 +185,6 @@ Java_gnu_java_awt_peer_gtk_GtkToolkit_gtkInit (JNIEnv *env,
   old_glog_func = g_log_set_default_handler (&glog_func, NULL);
 #endif
 
-#if GTK_CAIRO
-  cp_gtk_graphics2d_init_jni ();
-#endif
-  cp_gtk_graphics_init_jni ();
   cp_gtk_button_init_jni ();
   cp_gtk_checkbox_init_jni ();
   cp_gtk_choice_init_jni ();
@@ -391,6 +387,37 @@ Java_gnu_java_awt_peer_gtk_GtkToolkit_getScreenResolution
   return res;
 }
 
+/**
+ * Report the number of mouse buttons
+ * Returns the number of buttons of the first mouse found, or -1 if no mouse
+ * seems to be connected.
+ */
+JNIEXPORT jint JNICALL 
+Java_gnu_java_awt_peer_gtk_GtkToolkit_getMouseNumberOfButtons
+  (JNIEnv *env __attribute__((unused)), jobject obj __attribute__((unused)))
+{
+  jint res = -1;
+  GList *devices;
+  GdkDevice *d;
+
+  gdk_threads_enter ();
+
+  /* FIXME: Why doesn't this return the correct number? */
+  devices = gdk_devices_list();
+
+  while( res == -1 && devices != NULL )
+    {
+      d = GDK_DEVICE( devices->data );
+      if( d->source == GDK_SOURCE_MOUSE )
+	res = d->num_keys;
+      devices = devices->next;
+    }
+
+  gdk_threads_leave ();
+
+  return res;
+}
+
 #define CONVERT(type, state) \
   gdk_color_to_java_color (style->type[GTK_STATE_ ## state])
 
