@@ -1,5 +1,5 @@
 /* VMInetAddress.c - Native methods for InetAddress class
-   Copyright (C) 1998, 2002, 2005  Free Software Foundation, Inc.
+   Copyright (C) 1998, 2002, 2005, 2006  Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -41,7 +41,6 @@ exception statement from your version. */
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
-#include <assert.h>
 
 #include <jni.h>
 #include <jcl.h>
@@ -69,9 +68,6 @@ Java_java_net_VMInetAddress_getLocalHostname (JNIEnv * env,
   int result;
   jstring retval;
 
-  assert (env != NULL);
-  assert ((*env) != NULL);
-
 #ifndef WITHOUT_NETWORK
   TARGET_NATIVE_NETWORK_GET_HOSTNAME (hostname, sizeof (hostname), result);
   if (result != TARGET_NATIVE_OK)
@@ -100,9 +96,6 @@ Java_java_net_VMInetAddress_lookupInaddrAny (JNIEnv * env,
   jarray IParray;
   jbyte *octets;
 
-  assert (env != NULL);
-  assert ((*env) != NULL);
-
   /* Allocate an array for the IP address */
   IParray = (*env)->NewByteArray (env, 4);
   if (IParray == NULL)
@@ -150,9 +143,6 @@ Java_java_net_VMInetAddress_getHostByAddr (JNIEnv * env,
   int result;
   jstring retval;
 
-  assert (env != NULL);
-  assert ((*env) != NULL);
-
   /* Grab the byte[] array with the IP out of the input data */
   len = (*env)->GetArrayLength (env, arr);
   if (len != 4)
@@ -181,7 +171,8 @@ Java_java_net_VMInetAddress_getHostByAddr (JNIEnv * env,
 						 sizeof (hostname), result);
   if (result != TARGET_NATIVE_OK)
     {
-      JCL_ThrowException (env, UNKNOWN_HOST_EXCEPTION, "Bad IP address");
+      JCL_ThrowException (env, UNKNOWN_HOST_EXCEPTION,
+			  TARGET_NATIVE_LAST_ERROR_STRING ());
       return (jstring) NULL;
     }
 
@@ -214,9 +205,6 @@ Java_java_net_VMInetAddress_getHostByName (JNIEnv * env,
   jarray ret_octets;
   int max_addresses;
 
-  assert (env != NULL);
-  assert ((*env) != NULL);
-
   /* Grab the hostname string */
   hostname = (*env)->GetStringUTFChars (env, host, 0);
   if (!hostname)
