@@ -38,6 +38,9 @@ exception statement from your version. */
 
 package org.omg.CORBA;
 
+import gnu.CORBA.Minor;
+import gnu.CORBA.OrbRestricted;
+
 import org.omg.CORBA.Any;
 import org.omg.CORBA.BAD_OPERATION;
 import org.omg.CORBA.ORB;
@@ -57,11 +60,6 @@ import org.omg.CORBA.portable.OutputStream;
 */
 public abstract class PolicyHelper
 {
-  /**
-   * The cached {@link Policy} typecode, computed once.
-   */
-  private static TypeCode typeCode;
-
   /**
    * Get the type code of the {@link Policy}.
    *
@@ -69,9 +67,7 @@ public abstract class PolicyHelper
    */
   public static TypeCode type()
   {
-    if (typeCode == null)
-      typeCode = ORB.init().create_interface_tc(id(), "Policy");
-    return typeCode;
+    return OrbRestricted.Singleton.create_interface_tc(id(), "Policy");
   }
 
   /**
@@ -99,7 +95,9 @@ public abstract class PolicyHelper
       }
     catch (ClassCastException ex)
       {
-        throw new BAD_OPERATION("Policy expected");
+        BAD_OPERATION bad = new BAD_OPERATION("Policy expected");
+        bad.minor = Minor.Any;
+        throw bad;
       }
   }
 
