@@ -38,6 +38,8 @@ exception statement from your version. */
 
 package org.omg.CosNaming.NamingContextExtPackage;
 
+import gnu.CORBA.OrbRestricted;
+
 import org.omg.CORBA.Any;
 import org.omg.CORBA.ORB;
 import org.omg.CORBA.TypeCode;
@@ -59,11 +61,6 @@ public abstract class AddressHelper
   private static String _id =
     "IDL:omg.org/CosNaming/NamingContextExt/Address:1.0";
 
-  /**
-   * The cached type code (string alias).
-   */
-  private static TypeCode typeCode = null;
-
   /**
    * Just extracts string from this {@link Any}.
    */
@@ -99,13 +96,12 @@ public abstract class AddressHelper
   /**
    * Return the "Address", alias of String, typecode.
    */
-  public static synchronized TypeCode type()
+  public static TypeCode type()
   {
-    if (typeCode == null)
-      {
-        typeCode = ORB.init().create_string_tc(0);
-        typeCode = ORB.init().create_alias_tc(id(), "Address", typeCode);
-      }
+    ORB orb = OrbRestricted.Singleton;
+    TypeCode typeCode;
+    typeCode = orb.create_string_tc(0);
+    typeCode = orb.create_alias_tc(id(), "Address", typeCode);
     return typeCode;
   }
 
