@@ -39,6 +39,8 @@ exception statement from your version. */
 package org.omg.DynamicAny.DynAnyPackage;
 
 import gnu.CORBA.EmptyExceptionHolder;
+import gnu.CORBA.Minor;
+import gnu.CORBA.OrbRestricted;
 
 import org.omg.CORBA.Any;
 import org.omg.CORBA.BAD_OPERATION;
@@ -57,23 +59,13 @@ import org.omg.CORBA.portable.OutputStream;
 public abstract class TypeMismatchHelper
 {
   /**
-   * The cached typecode value, computed only once.
-   */
-  private static TypeCode typeCode;
-
-  /**
-   * Create the TypeMismatch typecode (structure,
-   * named "TypeMismatch").
+   * Create the TypeMismatch typecode (structure, named "TypeMismatch").
    */
   public static TypeCode type()
   {
-    if (typeCode == null)
-      {
-        ORB orb = ORB.init();
-        StructMember[] members = new StructMember[ 0 ];
-        typeCode = orb.create_exception_tc(id(), "TypeMismatch", members);
-      }
-    return typeCode;
+    ORB orb = OrbRestricted.Singleton;
+    StructMember[] members = new StructMember[0];
+    return orb.create_exception_tc(id(), "TypeMismatch", members);
   }
 
   /* Every user exception with no user defined
@@ -107,6 +99,7 @@ public abstract class TypeMismatchHelper
       {
         BAD_OPERATION bad = new BAD_OPERATION("TypeMismatch expected");
         bad.initCause(cex);
+        bad.minor = Minor.Any;        
         throw bad;
       }
   }
